
    processor 6502
    include vcs.h
    include macro.h

;-------------------------Constants Below---------------------------------

PADDLEHEIGHT    =   15; 20
BALLHEIGHT  =   3
DIGITHEIGHT =   16
PLAYHEIGHT  =   184
DIGITY      =   84

; Ball goes from 0 to 182
; Paddle goes from 01 to 169 (184 - PADDLEHEIGHT)


;-------------------------COLOR CONSTANTS (NTSC)--------------------------

GRAY        =   $00
GOLD        =   $10
ORANGE      =   $20
BURNTORANGE =   $30
RED     =   $40
PURPLE      =   $50
PURPLEBLUE  =   $60
BLUE        =   $70
BLUE2       =   $80
LIGHTBLUE   =   $90
TURQUOISE   =   $A0
GREEN       =   $B0
BROWNGREEN  =   $C0
TANGREEN    =   $D0
TAN     =   $E0
BROWN       =   $F0

;--------------------------TIA CONSTANTS----------------------------------

    ;--NUSIZx CONSTANTS
    ;   player:
ONECOPYNORMAL       =   $00
TWOCOPIESCLOSE      =   $01
TWOCOPIESMED        =   $02
THREECOPIESCLOSE    =   $03
TWOCOPIESWIDE       =   $04
ONECOPYDOUBLE       =   $05
THREECOPIESMED      =   $06
ONECOPYQUAD     =   $07
    ;   missile:
SINGLEWIDTHMISSILE  =   $00
DOUBLEWIDTHMISSILE  =   $10
QUADWIDTHMISSILE    =   $20
OCTWIDTHMISSILE     =   $30

    ;---CTRLPF CONSTANTS
    ;   playfield:
REFLECTEDPF     =   %00000001
SCOREPF         =   %00000010
PRIORITYPF      =   %00000100
    ;   ball:
SINGLEWIDTHBALL     =   SINGLEWIDTHMISSILE
DOUBLEWIDTHBALL     =   DOUBLEWIDTHMISSILE
QUADWIDTHBALL       =   QUADWIDTHMISSILE
OCTWIDTHBALL        =   OCTWIDTHMISSILE

    ;---HMxx CONSTANTS
LEFTSEVEN       =   $70
LEFTSIX         =   $60
LEFTFIVE        =   $50
LEFTFOUR        =   $40
LEFTTHREE       =   $30
LEFTTWO         =   $20
LEFTONE         =   $10
NOMOVEMENT      =   $00
RIGHTONE        =   $F0
RIGHTTWO        =   $E0
RIGHTTHREE      =   $D0
RIGHTFOUR       =   $C0
RIGHTFIVE       =   $B0
RIGHTSIX        =   $A0
RIGHTSEVEN      =   $90
RIGHTEIGHT      =   $80

    ;---AUDCx CONSTANTS (P Slocum's naming convention)
SAWSOUND        =   1
ENGINESOUND     =   3
SQUARESOUND     =   4
BASSSOUND       =   6
PITFALLSOUND        =   7
NOISESOUND      =   8
LEADSOUND       =   12
BUZZSOUND       =   15

    ;---SWCHA CONSTANTS (JOYSTICK)
J0RIGHT     =   %10000000
J0LEFT      =   %01000000
J0DOWN      =   %00100000
J0UP        =   %00010000
J1RIGHT     =   %00001000
J1LEFT      =   %00000100
J1DOWN      =   %00000010
J1UP        =   %00000001

    ;---SWCHB CONSTANTS (CONSOLE SWITCHES)
P1DIFF      =   %10000000
P0DIFF      =   %01000000
BWCOLOR     =   %00001000
SELECT      =   %00000010
RESET       =   %00000001

;-------------------------End Constants-----------------------------------

;-----------------------------Macros--------------------------------------

    MAC FILLER
        REPEAT {1}
        .byte {2}
        REPEND
    ENDM


  MAC DEBUG_BRK
    IF DEBUG
      brk                         ;
    ENDIF
  ENDM

  MAC CHECKPAGE
    IF >. != >{1}
      ECHO ""
      ECHO "ERROR: different pages! (", {1}, ",", ., ")"
      ECHO ""
      ERR
    ENDIF
  ENDM


  MAC BIT_B
    .byte   opBIT_B
  ENDM

  MAC BIT_W
    .byte   opBIT_W
  ENDM

;------------------------------Variables----------------------------------

    SEG.U Variables
    org $80


Counter         ds 1


DigitLPtr       ds 2
DigitRPtr       ds 2

DigitLTemp      ds 1
DigitRTemp      ds 1

PaddleLTemp     ds 1
PaddleRTemp     ds 1

PaddleLY        ds 1
PaddleRY        ds 1

PaddleValue     ds 1

BallTemp        ds 1

BallXfrac       ds 1
BallX           ds 1
BallYfrac       ds 1
BallY           ds 1

BallVelXfrac    ds 1
BallVelX        ds 1
BallVelYfrac    ds 1
BallVelY        ds 1


ScoreL          ds 1
ScoreR          ds 1

soundPointer    ds 2
sfxPlay         ds 1
sfxLast         ds 1

gameState       ds 1
attractLockout  ds 1

;-------------------------End Variables-----------------------------------

    SEG Bank0
    org $F000

Start
    CLEAN_START

;--any initial setup

    lda #REFLECTEDPF|SINGLEWIDTHBALL
    sta CTRLPF

    lda #ONECOPYQUAD|DOUBLEWIDTHMISSILE
    sta NUSIZ0
    lda #ONECOPYQUAD
    sta NUSIZ1

    lda #GRAY+14        ;white
    sta COLUPF
    lda #GRAY+9         ;gray
    sta COLUP0
    sta COLUP1



    lda #$80
    sta BallVelYfrac
    lda #$00
    sta BallVelY

    lda #$00
    sta BallVelXfrac
    lda #$01
    sta BallVelX

    lda #1
    sta VDELBL
    sta VDELP0

    lda #78
    sta BallX

    lda #79         ;net
    ldx #4
    jsr PositionASprite

    lda #14         ;L digit
    ldx #0
    jsr PositionASprite

    lda #90        ;R digit
    ldx #1
    jsr PositionASprite

    lda #128
    sta BallYfrac
    lda #80
    sta BallY

    lda #20
    sta attractLockout

;-------------------------------------------------------------------------
;--------------GAME MAIN LOOP---------------------------------------------
;-------------------------------------------------------------------------

MainGameLoop

    jsr VBLANKRoutine
    lda sfxPlay         ; NB - this is here to keep the display in check
    bne .nofill         ; if sfxPlay is not true we need to burn a line of the display
    sta WSYNC           ; hence this WSYNC. I stole a line from the start of the Kernel Routine
.nofill                 ; TODO - fix this so it doesn't rely on side effects
    jsr SFXTick
    jsr KernelRoutine
    jsr SFXTick
    jsr OverscanRoutine
    jmp MainGameLoop

;-------------------------------------------------------------------------
;-------------------VBLANK Routine----------------------------------------
;-------------------------------------------------------------------------

VBLANKRoutine
    lda #$82
    sta VBLANK          ;for paddles

    lda #%00001111
VSYNCLoop
    sta WSYNC
    sta VSYNC
    lsr
    bcs VSYNCLoop

    lda #43
    sta TIM64T

    jsr UpdateCounters
    jsr SetupDisplayVariables
    jsr MoveBall

    sta CXCLR

WaitForVblankEnd
    lda INTIM
    bne WaitForVblankEnd

    sta WSYNC
    sta VBLANK  ;turn off VBLANK - it was turned on by overscan

    rts

;-------------------------------------------------------------------------
;----------------------Kernel Routine-------------------------------------
;-------------------------------------------------------------------------

    ;--Notes on kernel:
    ;   left and right "paddles" are drawn with PF0!
    ;   left and right score digits are drawn with P0 (VDELed) and P1
    ;   center "net" is drawn with BL (VDELed)
    ;   ball is drawn with M0
    ;
    ;   Two-line kernel
    ;       Every line:
    ;           ball is drawn
    ;           "paddles" are drawn
    ;           paddle (INPT0 or INPT1, alternating frames,
    ;               indexed with X) is read
    ;       Every other line:
    ;           "net" is drawn
    ;           left and right score digits are drawn

    align 256

KernelRoutine
    sta WSYNC
    lda #$00
    sta PF0
    sta PF1
    sta PF2             ;+11    11  draw horizontal line at top of screen
    sta WSYNC
;    sta WSYNC          ; NB - this line was stolen for the main game loop to process an extra SFX tick

    ldy #PLAYHEIGHT     ;+2  2

    lda Counter
    and #1
    tax                 ;+7  9

    lda #255
    sta PaddleValue     ;+5 14  necessary to preload PaddleValue

    SLEEP 45            ;+45    59


    lda #BALLHEIGHT
    dcp BallTemp        ; dec BallTemp cmp BallTemp
    sbc #BALLHEIGHT-2
    sta ENAM0           ;+12    71  draw ball - a little early here.  Oh well.  Quick hack.

    lda #0              ;+2 73
    sta PF2             ;+3 76

;   sta WSYNC
    sta PF0
    sta PF1             ;+6  6  erase horizontal line at top of screen
KernelLoop
    lda #PADDLEHEIGHT
    dcp PaddleLTemp     ; dec PaddleLTemp cmp PaddleLTemp (5 cycles)
    sbc #<(PADDLEHEIGHT-64)
    and #$40
    sta PF0             ;+14    20  draw L "paddle"

    lda #DIGITHEIGHT-1  ;  2
    dcp DigitLTemp      ;  6    dec DigitLTemp cmp DigitLTemp
    bcs DoDrawDigitL    ;  2/3
    lda #0              ;
    .byte $2C           ;  4 (but covers the next five)
DoDrawDigitL
    lda (DigitLPtr),Y   ;  5
    sta GRP0            ;+18    38  draw L score.  VDELed.  Could use SkipDraw here
                        ;       and save 1 cycle.


    lda INPT0,X         ; 4
    bpl ReadPaddle1     ; 2/3
    .byte $2C           ; 4
ReadPaddle1
    sty PaddleValue     ; 3  (4+3+3) or (4+2+4) =  +10    48  read paddle

    lda #PADDLEHEIGHT
    dcp PaddleRTemp     ; dec PaddleRTemp cmp PaddleRTemp
    sbc #<(PADDLEHEIGHT-64)
    and #$40
    sta PF0             ;+14    62  draw right "paddle"

    dey                 ;+2 64

    lda #BALLHEIGHT
    dcp BallTemp        ; dec BallTemp cmp BallTemp
    sbc #BALLHEIGHT-2
    sta ENAM0           ;+12    76  draw ball

    lda #PADDLEHEIGHT
    dcp PaddleLTemp     ;dec PaddleLTemp cmp PaddleLTemp
    sbc #<(PADDLEHEIGHT-64)
    and #$40
    sta PF0             ;+14    14  draw left "paddle"

    lda #DIGITHEIGHT-1
    dcp DigitRTemp      ; dec DigitRtemp cmp DigitRTemp
    bcs DoDrawDigitR
    lda #0
    .byte $2C
DoDrawDigitR
    lda (DigitRPtr),Y
    sta GRP1            ;+18    32  draw right score digit
                        ;       necessary to use DoDraw (or similar)
                        ;       here - because of VDEL GRP1 must be
                        ;       written to every time.

    lda INPT0,X
    bpl ReadPaddle2
    .byte $2C
ReadPaddle2
    sty PaddleValue     ;+10    42  read paddle

    lda #PADDLEHEIGHT
    dcp PaddleRTemp     ; dec PaddleRTemp cmp PaddleRTemp
    sbc #<(PADDLEHEIGHT-64)
    and #$40
    sta PF0             ;+14    56  draw right "paddle"

    tya
    lsr
    sta ENABL           ;+7 63  VDELed - this is the 'net'

    nop                 ;+2 65  Two free cycles

    lda #BALLHEIGHT
    dcp BallTemp        ; dec BallTemp cmp BallTemp
    sbc #BALLHEIGHT-2
    sta ENAM0           ;+12     1  draw ball

    dey
    bne KernelLoop      ;+5  6

    lda #$00
    sta PF0
    sta PF1
    sta PF2             ;       draw horizontal line at bottom of screen



    sta WSYNC
    lda #0
    sta ENAM0
    sta ENABL
    sta GRP0
    sta GRP1            ;+14    14  erase all objects (remember that BL and P0 are VDELed)
    sta WSYNC
    sta WSYNC
    lda #0
    sta PF0
    sta PF1
    sta PF2             ;       erase horizontal line at bottom of screen
    sta WSYNC
    rts

;-------------------------------------------------------------------------
;------------------------Overscan Routine---------------------------------
;-------------------------------------------------------------------------

OverscanRoutine
    lda #2
    sta WSYNC
    sta VBLANK  ;turn on VBLANK
    lda  #34
    sta  TIM64T

    ; check collisions first because paddle value will be correct
    jsr CheckCollisions
    jsr SetPaddleY

WaitForOverscanEnd
    lda INTIM
    bne WaitForOverscanEnd
    rts

;-------------------------------------------------------------------------
;----------------------------End Main Routines----------------------------
;-------------------------------------------------------------------------


;*************************************************************************

;-------------------------------------------------------------------------
;----------------------Begin Subroutines----------------------------------
;-------------------------------------------------------------------------

CheckCollisions subroutine
    ;--ball to paddle collision is actually a M0-to-PF collision.  Will have to look at ball X
    ;   position to determine which paddle is hit.

    lda CXM0FB      ;M0 to PF in bit 7
    asl
    bcc NoBallToPaddleCollision

    ;--so, we hit a paddle

    lda BallX
    cmp #80
    bcs .rightSide

    lda BallVelX
    bpl .exit               ; ball already heading correct direction (prevent double trigger)

    lda #$00
    sta BallVelXfrac
    lda #$01
    sta BallVelX
    ldx #00                 ; left paddle
    beq .checkbounce

.rightSide:
    lda BallVelX
    bmi .exit               ; ball already heading correct direction (prevent double trigger)

    lda #$00
    sta BallVelXfrac
    lda #$ff
    sta BallVelX
    ldx #01                 ; right paddle

.checkbounce
    lda BallY
    clc
    adc #4
    sec
    sbc PaddleLY,X

    ; clamp to 0-19
    bpl .goodl
    lda #00
.goodl
    cmp #20
    bcc .goodh
    lda #19
.goodh

    tax
    lda newvelYl,x
    sta BallVelYfrac
    lda newvelYh,x
    sta BallVelY

    lda #<PADDLE_HIT
    sta soundPointer
    lda #>PADDLE_HIT
    sta soundPointer+1
    sta sfxPlay             ; starts the SFX

.exit

NoBallToPaddleCollision

    rts

newvelYl:    .byte $00,$00,$00, $00,$00,$00, $80,$80,$80, $00, $80,$80,$80, $00,$00,$00, $00,$00,$00, $00
newvelYh:    .byte $fe,$fe,$fe, $ff,$ff,$ff, $ff,$ff,$ff, $00, $00,$00,$00, $01,$01,$01, $02,$02,$02, $02

;-------------------------------------------------------------------------

SetPaddleY subroutine

    lda Counter
    and #1
    tax

    lda PaddleValue
    cmp #PLAYHEIGHT-PADDLEHEIGHT
    bcc PaddleNotTooHigh
    lda #PLAYHEIGHT-PADDLEHEIGHT
;    clc
PaddleNotTooHigh
;    adc PaddleLY,X
;    ror
    sta PaddleLY,X
    rts

;-------------------------------------------------------------------------

MoveBall subroutine
    lda BallVelXfrac
    clc
    adc BallXfrac
    sta BallXfrac
    lda BallX
    adc BallVelX
    sta BallX

    lda BallVelYfrac
    clc
    adc BallYfrac
    sta BallYfrac
    lda BallY
    adc BallVelY
    sta BallY

    ;--check boundaries
    ;--if hit any edge, reverse that direction
    ;--if hit left/right edge, increase score of other player
    lda BallX
    cmp #160
    bcc BallOnScreenX

    ; play off screen sound effect
    lda #<MISSED_BALL
    sta soundPointer
    lda #>MISSED_BALL
    sta soundPointer+1
    sta sfxPlay             ; starts the SFX

    ; reset ball position
    lda #128
    sta BallXfrac
    lda #80
    sta BallX

    lda BallVelX
    bpl BallOffRightEdge

    ;--ball off left edge
    ;--increase right score
    lda ScoreR
    clc
    adc #$01
    cmp #12
    bcc .goodscr
    lda #00
.goodscr
    sta ScoreR
    jmp BallOnScreenX

BallOffRightEdge

    ;--ball off right edge
    ;--increase left score
    lda ScoreL
    clc
    adc #$01
    cmp #12
    bcc .goodscl
    lda #00
.goodscl
    sta ScoreL

BallOnScreenX
    ;--now check top and bottom bounds
    lda BallY
    cmp #PLAYHEIGHT-2
    bcc BallOnScreenY

    ;--ball hit top or bottom

    ; play wall hit sound effect
    lda #<WALL_HIT
    sta soundPointer
    lda #>WALL_HIT
    sta soundPointer+1
    sta sfxPlay             ; starts the SFX

    ; invert Y velocity (2s complement)
    lda BallVelYfrac
    eor #$FF
    clc
    adc #$01
    sta BallVelYfrac
    lda BallVelY
    eor #$FF
    adc #$00
    sta BallVelY

BallOnScreenY

    lda BallX
    ldx #2
    jsr PositionASprite

    rts

;-------------------------------------------------------------------------

SetupDisplayVariables subroutine

    lda #PLAYHEIGHT+1
    sec
    sbc PaddleLY
    sta PaddleLTemp

    lda #PLAYHEIGHT+1
    sec
    sbc PaddleRY
    sta PaddleRTemp

    lda #PLAYHEIGHT
    sec
    sbc BallY
    sta BallTemp

    lda #PLAYHEIGHT
    lsr
    sec
    sbc #DIGITY
    clc
    adc #DIGITHEIGHT
    sta DigitLTemp

    lda #PLAYHEIGHT
    lsr
    sec
    sbc #DIGITY
    clc
    adc #DIGITHEIGHT
    sta DigitRTemp


    lda ScoreL
    and #$0F
    asl
    tay
    lda DigitTable,Y
    sec
    sbc #1


;   lda #<Digit3-1
    sec
    sbc #DIGITY*2
    clc
    adc #DIGITHEIGHT*2
    sta DigitLPtr
    lda DigitTable+1,Y
    sta DigitLPtr+1


    lda ScoreR
    and #$0F
    asl
    tay
    lda DigitTable,Y

;   lda #<Digit3
    sec
    sbc #DIGITY*2
    clc
    adc #DIGITHEIGHT*2
    sta DigitRPtr
    lda DigitTable+1,Y
    sta DigitRPtr+1
    rts

;-------------------------------------------------------------------------

UpdateCounters subroutine

    dec Counter

    rts

;-------------------------------------------------------------------------

    align 256

PositionASprite ;call this function with A == horizontal position (0-159)
                ;and X == the object to be positioned (0=P0, 1=P1, 2=M0, etc.)
                ;This function will change A, which
                ;will be the value put into HMxx when returned.
                ;Call this function with at least 14 cycles left in the scanline
                ;(jsr + sec + sta WSYNC + sta HMCLR = 14); it will return 9 cycles
                ;into the second scanline
    sec
    sta HMCLR
    sta WSYNC           ;begin line 1
DivideLoop
    sbc #15
    bcs DivideLoop          ;+4/5    4/ 9.../54

    eor #7              ;+2  6/11.../56
    asl
    asl
    asl
    asl             ;+8 14/19.../64

    sta.wx HMP0,X           ;+5 19/24.../69

    sta RESP0,X         ;+4 23/28/33/38/43/48/53/58/63/68/73
    sta WSYNC           ;+3  0  begin line 2
    sta HMOVE           ;+3
Return                          ;label for time-wasting 'jsr's

    rts


;----------------------------------------------------------------------------

;;;;;;;;;; SFX code ;;;;;;;;;;

SFXTick         subroutine

                lda sfxPlay
                beq ExitSFX

                ldy #$00

                lda (soundPointer),y
                cmp sfxLast
                beq .silence
                sta sfxLast
                tax
                beq .silence

                sta AUDF0
                lsr
                lsr
                lsr
                lsr
                lsr
                tay
                lda soundTypeArray,y
                sta AUDC0
.silence
                ldy #$01
                lda (soundPointer),y
                sta AUDV0
                ora sfxLast
                sta sfxPlay

                inc soundPointer
                inc soundPointer
                bne .incrementdone
                inc soundPointer+1
.incrementdone
                rts

ExitSFX         lda #0
                sta AUDV0
                rts


;*************************************************************************

;-------------------------------------------------------------------------
;-------------------------Data Below--------------------------------------
;-------------------------------------------------------------------------


Z_LEAD_B3       = $B4
Z_SQUARE_B4_1   = $1E

soundTypeArray
    byte SQUARESOUND,BASSSOUND,PITFALLSOUND,NOISESOUND,BUZZSOUND,LEADSOUND,SAWSOUND,ENGINESOUND
    ;byte 4,6,7,8,15,12,1,3


        align 2

; Ok, the SFX data is stored as audio data byte followed by envelope byte.
; The audio data byte is in Manuel R. format with the top 3 bits index a table to setup the timbre in AUDCx
; and the bottom 5 bits go to setup the frequency in AUDFx
; The envelope byte is just the volume for AUDCx and is done at 120Hz (8.3 ms period)
; (actually it's about 70/30 so it's like 11.6 ms and then 5 ms instead of 8.3/8.3)

; This could be a lot more efficient I realize, but it's just quick and dirty to get the envelope stuff working
MISSED_BALL

    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #8
    .byte #Z_LEAD_B3, #15
    .byte 0,0

;        align 2
;
WALL_HIT
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #15
    .byte #Z_LEAD_B3, #15
    .byte 0,0
;
;
;        align 2
PADDLE_HIT
    .byte #Z_SQUARE_B4_1, #10
    .byte #Z_SQUARE_B4_1, #15
    .byte 0,0


DigitTable
    .word Digit0,Digit1,Digit2,Digit3,Digit4,Digit5,Digit6,Digit7,Digit8,Digit9
    .word Digit10,Digit11

; Note all of this digit image data is setup so that the indexing from the kernel
; doesn't cross a page boundary and result in an extra cycle (which would glitch the display)

        org $FB90

Digit0
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
Digit1
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
Digit2
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--

        CHECKPAGE Digit0

        org $FC90

Digit3
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--

Digit4
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
Digit5
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--

enddigit5

        CHECKPAGE Digit3

        org $FD90

Digit6
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--
        .byte 0,#%00000100;--

Digit7
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--

Digit8:
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
enddigit8

        CHECKPAGE Digit6

        org $FE90

Digit9:
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000001;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000101;--
        .byte 0,#%00000111;--
        .byte 0,#%00000111;--

Digit10
        .byte 0,#%00100111;--
        .byte 0,#%00100111;--
        .byte 0,#%00100101;--
        .byte 0,#%00100101;--
        .byte 0,#%00100101;--
        .byte 0,#%00100101;--
        .byte 0,#%00100101;--
        .byte 0,#%00100101;--
        .byte 0,#%00100101;--
        .byte 0,#%00100101;--
        .byte 0,#%00100101;--
        .byte 0,#%00100101;--
        .byte 0,#%00100101;--
        .byte 0,#%00100101;--
        .byte 0,#%00100111;--
        .byte 0,#%00100111;--
Digit11
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
        .byte 0,#%00100001;--
enddigit11

        CHECKPAGE Digit9

;-------------------------------------------------------------------------
;-------------------------End Data----------------------------------------
;-------------------------------------------------------------------------

 ORG $FFF8
 .word 0        ; for supercharger
 .word 0        ; nmi vector
 .word Start    ; start vector
 .word Start    ; brk vector


